//
//  MINTLCPAbstructSourceItem.swift
//  TextEditDoc
//
//  Created by narita on 2018/11/11.
//  Copyright © 2018 narita. All rights reserved.
//

import Cocoa

class MINTLCPAbstructSourceItem: NSObject, NSCopying
{
    // MARK: - class methods
    
    // MARK: - init methods

    init(description: String = "", action: @escaping () -> Void, isGroup: Bool = false)
    {
        self.string = description
        self.action = action
        self.isGroup = isGroup
    }

    convenience init(groupDescription: String, kindTag: String)
    {
        self.init(description:groupDescription, action:
        {
            let mgr = MINTLCPManager.shared
            
            if !mgr.commandString.contains(kindTag)
            {
                let theNewString =  (mgr.commandString as String) + " " + kindTag
                
                mgr.willChangeValue(for: \.commandString)
                mgr.commandString = theNewString as! NSMutableString
                mgr.didChangeValue(for: \.commandString)

                mgr.keyEvent(nil)
            }
        })
        
        self.isGroup = true
    }

    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    func copy(with zone: NSZone? = nil) -> Any
    {
        return MINTLCPAbstructSourceItem(
                                            description: self.string,
                                            action: self.action,
                                            isGroup: self.isGroup )
    }

    
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    @objc func performAction()
    {
        if !self.isGroup
        {
            MINTLCPManager.shared.closePanel(nil)
        }
        self.action()
    }
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    // Table へ表示
    @objc var string:   String = ""    // 表示文字列
    
    @objc var isGroup:  Bool   = false // グループか否か
    @objc var action: (() -> Void) // 実行
    
    // Tableで表示するための文字列。マッチした部分を強調表示(Bold)にする
    @objc var mathcedString: NSAttributedString = NSAttributedString(string: "")
    
    
    @objc var displayString: NSAttributedString
    {
        if mathcedString.length == 0
        {
            return NSAttributedString(string: string)
        }
                
        return mathcedString
    }

    // MARK: - Utility methods

}
