//
//  MINTLCPManager.swift
//  TextEditDoc
//
//  Created by narita on 2018/06/07.
//  Copyright © 2018年 narita. All rights reserved.
//

import Cocoa

class MINTLCPManager: NSObject
{
    // MARK: - class methods
    
    static let shared: MINTLCPManager =
    {
        return MINTLCPManager()
    }()
    
    // MARK: - init methods
    
    override private init()
    {
        super.init()
        
        // NSApplicationのNotificationを取得する
        let theNC = NotificationCenter.default
        
        theNC.addObserver(self,
                          selector: #selector(self.applicationWillTerminateNotification),
                          name: NSApplication.willTerminateNotification,
                          object: NSApp)
        
        
        self.commandStringObservation = self.observe(\.commandString, options:[.new, .old])
        {
            _, _ in
            
            self.keyEvent(nil)
        }        
    }
    
    var commandStringObservation: NSKeyValueObservation? = nil

    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    @objc func applicationWillTerminateNotification()
    {
        self.terminate(nil)
    }

    // MARK: - action methods
    
    func updateRecodeArray( searchStrings: [NSString], kindStrings:[NSString])
    {
        let theSources: Array<MINTLCPAbstructSource>

        // kindがあればkindでor結合でフィルタリング
        if kindStrings.count == 0
        {
            theSources = self.sources
        }
        else
        {
            let theGrepString = (kindStrings as NSArray).componentsJoined(by:"|")
            
            theSources = self.sources.filter()
            {
                theGrepString.regularExpressionWithIgnoreCaseMatches(string: $0.kindStringWithTag)
            }
        }
        
        // サーチワードがあれば、サーチワードでフィルタリング and 結合
        let theArray = theSources.reduce([])
        {
            var theArray = Array<MINTLCPAbstructSourceItem>()
            
            let theSourceItems: Array<MINTLCPAbstructSourceItem> =
                (searchStrings.count == 1 && searchStrings.first == "") ?
                    $1.sourceItems : $1.sourceItems(strings: searchStrings)
            
            // グループ名の行を追加する。
            if theSourceItems.count != 0
            {
                // ToDo: 選択した時のアクションの機能を追加する
                theArray.append(MINTLCPAbstructSourceItem(groupDescription:$1.groupDisplayName,
                                                                      kindTag: $1.kindStringWithTag))
                
                theArray.append(contentsOf: theSourceItems)
            }
            
            return $0 + theArray
        }

        // KVOで知らせる
        self.mutableArrayValue(forKey:"recodeArray").setArray(theArray)
    }

    
    
    func parseCommadnString() -> ( searchStrings: [NSString], kindStrings: [NSString])
    {
        let theCommandString = self.commandString.trimmingCharacters(in: NSCharacterSet.whitespaces)
        
        // kind:xxxx などのcommandその物と、検索文字列の分離
        // 空白(あるいはそれに相当するもので分離)全角の空白でも分離する
        let theSeparatedCmdStrings = theCommandString.components(separatedBy: NSCharacterSet.whitespaces)

        var theKindStrings: [NSString] = []
        var theSearchStrings: [NSString] = []
        
        theSeparatedCmdStrings.forEach()
        {
            if "^kind:.*".regularExpressionWithIgnoreCaseMatches(string: $0 as String)
            {
                theKindStrings.append($0 as NSString)
            }
            else
            {
                theSearchStrings.append($0 as NSString)
            }
        }
        
        return ( searchStrings: theSearchStrings, kindStrings: theKindStrings )
    }
    
    
    // keyイベント毎に呼び出される。
    @IBAction func keyEvent(_ sender: Any?)
    {
        let theResult = self.parseCommadnString()
        
        self.updateRecodeArray(searchStrings: theResult.searchStrings, kindStrings: theResult.kindStrings)
    }

    @IBAction func active(_ sender: Any?)
    {
        // firstResponder, keyResponderを保存

        self.sources.forEach()
        {
            $0.active()
        }
        
        self.showPanel(nil)
    }
    
    @IBAction func deactive(_ sender: Any?)
    {
        self.closePanel(nil)

        // 保存していたfirstResponder, keyResponderを復帰する

        self.sources.forEach()
        {
            $0.deactive()
        }
    }

    @IBAction func launching(_ sender: Any?)
    {
        self.sources.forEach()
        {
            $0.launching()
        }
    }

    @IBAction func terminate(_ sender: Any?)
    {
        self.sources.forEach()
        {
            $0.terminate()
        }
    }
    
    // panelを今すぐ表示
    @IBAction func showPanel(_ sender: Any?)
    {
        if let thePanelController = self.panelController
        {
            if panel!.isVisible == false
            {
                // firstResponderの保存(予定)
                
                thePanelController.relayoutPanelPosition()
                thePanelController.showWindow(self)
            }
        }
    }
    
    // panelを今すぐclose & timerのcancelを行う
    @IBAction func closePanel(_ sender: Any?)
    {
        if let thePanelController = self.panelController
        {
            if panel!.isVisible == true
            {
                thePanelController.close()
                
                // firstResponderの復帰(予定)
            }
        }
    }
    
    // MARK: - event handling methods

    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @objc open var commandString: NSMutableString = ""
    
    @objc open var recodeArray: NSMutableArray? = NSMutableArray()
    
    @objc open lazy var sources: Array<MINTLCPAbstructSource> = Array<MINTLCPAbstructSource>()
    
    @objc open lazy var panelController: MINTLCPPanelController? =
    {
        return
            NSStoryboard.init(
                name:"MINTLCPPanelController",
                bundle:nil).instantiateInitialController() as! MINTLCPPanelController?
    }()

    @objc class func keyPathsForValuesAffectingPanel() -> Set<String>
    {
        return [ "panelController",
                 "panelController.panel" ]
    }
    
    @objc open var panel: MINTLCPPanel?
    {
        return self.panelController?.panel
    }

    @objc class func keyPathsForValuesAffectingViewController() -> Set<String>
    {
        return [ "panelController",
                 "panelController.viewController" ]
    }
    
    @objc open var viewController: MINTLCPViewController?
    {
        return self.panelController?.viewController
    }
    
    // MARK: - Utility methods

}


extension String
{
    func regularExpressionMatches(string: String) -> Bool
    {
        if let regex = try? NSRegularExpression(pattern: self, options: [])
        {
            let num = regex.numberOfMatches(in: string,
                                            options: [],
                                            range: NSMakeRange(0, string.count))
            
            return num > 0
        }
        
        return false
    }
    
    func regularExpressionWithIgnoreCaseMatches(string: String) -> Bool
    {
        if let regex = try? NSRegularExpression(pattern: self, options: [.caseInsensitive])
        {
            let num = regex.numberOfMatches(in: string,
                                            options: [],
                                            range: NSMakeRange(0, string.count))
            
            return num > 0
        }
        
        return false
    }

    
    func rangeOfFirstMatchsRegularExpressionWithIgnoreCaseMatches(string: String) -> NSRange
    {
        if let regex = try? NSRegularExpression(pattern: self, options: [.caseInsensitive])
        {
            let theRange = regex.rangeOfFirstMatch(in: string,
                                                   options: [],
                                                   range: NSMakeRange(0, string.count))
            
            return theRange
        }
        
        return NSRange()        
    }
}


